@@ -29,5 +29,11 @@ def load_function_by_name(name):
             logger.info("load user defined table function", function_name=name)
             return _find_base_function_from_lib(lib)
         except:
-            logger.info("function not found", function_name=name)
-            return None
+            try:
+                importlib.invalidate_caches()
+                lib = importlib.import_module(name)
+                logger.info("clean cache and load user defined table function", function_name=name)
+                return _find_base_function_from_lib(lib)
+            except Exception as e:
+                logger.error("function not found", function_name=name, error=str(e))
+                return None
