@@ -4,6 +4,10 @@ import argparse
 import sys
 import pkgutil
 import toml
+import shutil
+import time
+import subprocess
+import tarfile
 from stonewave.sql.udtfs.function_executor import execute
 from stonewave.sql.udtfs.constants import (
     USER_DEFINED_TABLE_FUNCTIONS_PATH,
@@ -49,10 +53,11 @@ def list_udtfs_cmd(args):
         print(json.dumps(udtfs_list, indent=4))
 
 
-def _validate_function(func_name):
-    sys.path.append("/tmp")
+def _validate_function(func_path, func_name):
+    sys.path.append(func_path)
     func = load_function_by_name(func_name)
     if func is None:
+        shutil.rmtree(func_path)
         raise Exception("Can not find method implements from BaseFunction in stonewave.sql.udtfs.base_function")
     func_dir = func().__dir__()
     if "get_name" not in func_dir or "process" not in func_dir:
@@ -65,48 +70,100 @@ def _read_signature_list(path):
     return info_toml[SIGNATURE_LIST]
 
 
+def _raise_register_exception(temporary_working_dir, msg):
+    shutil.rmtree(temporary_working_dir)
+    raise Exception(msg)
+
+
 def register_udtf(func_name, func_path, update=False):
-    # install package to temperary directory, to avoid installing an invalid function
-    os.system("pip install {} -t /tmp/{}".format(func_path, func_name))
-    if not os.path.exists("/tmp/{}".format(func_name)):
-        raise Exception("Failed to install python package: {}".format(func_name))
-    project_name = None
+    if os.path.exists(os.path.join(os.path.dirname(__file__), "functions", func_name)):
+        raise Exception("Should not update build-in table function: {}".format(func_name))
+
+    if not os.path.exists(func_path):
+        raise Exception("Failed to install python package: '{}' does not exist.".format(func_path))
+
+    if os.path.splitext(func_path)[1] != ".gz":
+        raise Exception(
+            "Failed to install python package: please use eggfarm to help develop new python table function"
+        )
+
+    temporary_working_dir = "/tmp/{}_{}".format(func_name, int(round(time.time() * 1000)))
+
+    # decompress tar file to temporary working dir
+    # os.mkdir(temporary_working_dir)
+    func_tar_file = tarfile.open(func_path)
+    func_tar_file.extractall(path=temporary_working_dir)
+    func_tar_file.close()
+
+    # install python table function
+    func_path = os.path.join(temporary_working_dir, func_name, func_name)
+    commands = [
+        "pip",
+        "install",
+        "--no-index",
+        "-f",
+        os.path.join(temporary_working_dir, func_name),
+        func_name,
+        "-t",
+        func_path,
+    ]
+    installation = subprocess.run(
+        commands,
+        stderr=subprocess.PIPE,
+    )
+
+    if installation.returncode != 0:
+        raise Exception("Failed to install python package: {}".format(installation.stderr.decode("utf-8")))
+
     try:
-        project_name = parse_wheel_filename(os.path.basename(func_path)).project
-    except InvalidFilenameError:
-        project_name = func_path
-    os.system("mv /tmp/{0}/{1}/* /tmp/{0} && rm -rf /tmp/{0}/{1}".format(func_name, project_name))
+        filelist = os.listdir(os.path.join(func_path, func_name))
+        for file in filelist:
+            src = os.path.join(func_path, func_name, file)
+            dst = os.path.join(func_path, file)
+            shutil.move(src, dst)
+        shutil.rmtree(os.path.join(func_path, func_name))
+    except Exception as e:
+        logger.error("project path name is not as same as package name", error_message=str(e))
+        _raise_register_exception("Please make sure project path name is as same as package name")
     if not update:
-        info_path = os.path.join("/tmp", func_name, USER_DEFINED_TABLE_FUNCTION_INFO_FILE)
+        if os.path.exists(os.path.join(USER_DEFINED_TABLE_FUNCTIONS_PATH, func_name)):
+            _raise_register_exception(temporary_working_dir, "Function already exists")
+        info_path = os.path.join(func_path, USER_DEFINED_TABLE_FUNCTION_INFO_FILE)
         if not os.path.exists(info_path):
             logger.error(
                 "can not load function information",
                 function_name=func_name,
                 info_path=USER_DEFINED_TABLE_FUNCTION_INFO_FILE,
             )
-            raise Exception("Please include info.toml for function information")
-        check_expected_parameters_list(_read_signature_list(info_path))
+            _raise_register_exception(temporary_working_dir, "Please include info.toml for function information")
+        try:
+            check_expected_parameters_list(_read_signature_list(info_path))
+        except Exception as e:
+            _raise_register_exception(temporary_working_dir, str(e))
     else:
         existing_info_path = os.path.join(
             USER_DEFINED_TABLE_FUNCTIONS_PATH, func_name, USER_DEFINED_TABLE_FUNCTION_INFO_FILE
         )
         if not os.path.exists(existing_info_path):
-            raise Exception("Should not update build-in table function '{}'".format(func_name))
+            _raise_register_exception(
+                temporary_working_dir, "User defined table function does not exists: {}".format(func_name)
+            )
         existing_signature_list = _read_signature_list(existing_info_path)
-        new_info_path = os.path.join("/tmp", func_name, USER_DEFINED_TABLE_FUNCTION_INFO_FILE)
+        new_info_path = os.path.join(func_path, USER_DEFINED_TABLE_FUNCTION_INFO_FILE)
         if not os.path.exists(new_info_path):
             logger.error(
                 "can not load function information",
                 function_name=func_name,
                 info_path=USER_DEFINED_TABLE_FUNCTION_INFO_FILE,
             )
-            raise Exception("Please include info.toml for function information")
+            _raise_register_exception(temporary_working_dir, "Please include info.toml for function information")
         new_signature_list = _read_signature_list(new_info_path)
-        if not existing_signature_list.equals(new_signature_list):
-            raise Exception("Should not update table function signature list")
+        if existing_signature_list != new_signature_list:
+            _raise_register_exception(temporary_working_dir, "Should not update table function signature list")
+    _validate_function(os.path.join(temporary_working_dir, func_name), func_name)
 
-    _validate_function(func_name)
-    os.system("mv /tmp/{0} {1}/{0}".format(func_name, USER_DEFINED_TABLE_FUNCTIONS_PATH))
+    shutil.copytree(func_path, os.path.join(USER_DEFINED_TABLE_FUNCTIONS_PATH, func_name), dirs_exist_ok=True)
+    shutil.rmtree(temporary_working_dir)
 
 
 def register_udtf_cmd(args):
