@@ -18,10 +18,10 @@ class SharedMemoryRecordBatchReader(object):
 
     def read(self):
         # TODO: figure out if pa.py_buffer(shm.buf) makes any difference
-        reader = pa.ipc.open_stream(self.shm.buf)
-        batches = list(reader)
-        assert len(batches) == 1
-        batch = batches[0]
+        with pa.ipc.open_stream(self.shm.buf) as reader:
+            batches = list(reader)
+            assert len(batches) == 1
+            batch = batches[0]
         if batch is not None:
             logger.debug(
                 "read record batch via shared memory",
