@@ -1,7 +1,7 @@
 from stonewave.sql.udtfs.constants import VALIDATE_ERROR_MSG_TEMPLATE
 from stonewave.sql.udtfs.constants import ParameterDataType as pdt
 from stonewave.sql.udtfs.constants import ParameterType as pt
-from stonewave.sql.udtfs.table_row_writer import TableRowWriter
+from stonewave.sql.udtfs.table_writer import TableWriter
 from stonewave.sql.udtfs.protocol.ipc import share_record_batch
 from stonewave.sql.udtfs.protocol.fsm.apply.apply_function import ApplyOperator
 import pyarrow as pa
@@ -93,8 +93,8 @@ def eval_table_function_test(func_method, params, expected_parameters_list):
     """
     _validate_eval_parameters(params, expected_parameters_list)
     func = func_method()
-    row_writer = TableRowWriter()
-    func.process(row_writer, 0, params)
+    row_writer = TableWriter()
+    func.process(params, row_writer, 0)
     batch = None
     begininig = True
     while begininig or batch is None:
@@ -115,10 +115,11 @@ def apply_table_function_test(func_method, left_table, is_outer_apply, params, e
     return result
 
 
-def check_batch_iterator_results_with_csv_file(batch_iterator, expected_result_csv_path):
+def get_table_from_batch_iterator(batch_iterator):
     """
-    make batch iterator to table and compared with csv file table
-    :param
+    get table from batch iterator
+    :param batch_iterator:
+    :return:
     """
     batches = []
     try:
@@ -126,6 +127,16 @@ def check_batch_iterator_results_with_csv_file(batch_iterator, expected_result_c
     except StopIteration:
         pass
     table = pa.Table.from_batches(batches)
+    return table
+
+
+def check_table_results_with_csv_file(table, expected_result_csv_path):
+    """
+    check table results with csv file
+    :param table:
+    :param expected_result_csv_path:
+    :return:
+    """
     # empty string cell will be read as null cell
     convert_options = csv.ConvertOptions(strings_can_be_null=True)
     expected_table = csv.read_csv(expected_result_csv_path, convert_options=convert_options)
